import styled, { css } from 'styled-components';
import { isEmpty } from 'lodash-es';
import theme from '../../../../theme';
import { colorUtil } from '../../../../utils/ColorUtil';
import { textUtil } from '../../../../utils/TextUtil';

export interface ParagraphProps {
  content?: string;
  size?: 'sm' | 'md' | 'lg';
  textAlign?: 'left' | 'center' | 'right';
  fontWeight?: 'normal' | 'bold';
  whiteSpace?: string;
  isMobile?: boolean;
  color?: string;
  partialColors?: string[];
  margin?: string;
  padding?: string;
  isVisible?: boolean;
  isFullWidth?: boolean;
}

function Paragraph(props: ParagraphProps) {
  const { size, content, isVisible, partialColors, ...remain } = props;
  const texts = textUtil.processTextWithNewlines(content);

  if (!isVisible) return null;
  return (
    <ParagraphStyled {...props}>
      {texts?.map((text, i) => {
        const replacedText = colorUtil.applyColorsToText(text, partialColors);

        /** 텍스트가 가공된 경우 */
        if (!isEmpty(replacedText)) {
          const innerHtml = { __html: replacedText };
          return (
            <>
              <span key={i} dangerouslySetInnerHTML={innerHtml} />
              <br />
            </>
          );
        }

        /** 텍스트가 가공되지 않은 경우 */
        return (
          <>
            <span key={i}>{text}</span>
            <br />
          </>
        );
      })}
    </ParagraphStyled>
  );
}

Paragraph.defaultProps = {
  content: 'Paragraph Paragraph Paragraph\nParagraph Paragraph Paragraph',
  size: 'md',
  color: theme.colors.black,
  partialColors: [theme.colors.black],
  fontWeight: 'normal',
  isMobile: false,
  isVisible: true,
  isFullWidth: false,
  textAlign: 'left',
};

export default Paragraph;

const ParagraphCommonStyles = css<ParagraphProps>`
  font-family: ${props => (props.fontWeight === 'bold' ? `${theme.fontFamily.bold}` : `${theme.fontFamily.normal}`)};
  font-weight: normal;
  color: ${props => props.color};
  line-height: 1.5em;
  margin: 0;
  text-align: ${props => props.textAlign};
  white-space: ${props => props.whiteSpace};
`;

export const smallFontStyles = css<ParagraphProps>`
  font-size: ${props => (!props.isMobile ? '16px' : '12px')};
  letter-spacing: ${props => (!props.isMobile ? '-0.3px' : '-0.2px')};
  line-height: 1.25;
`;

export const mediumFontStyles = css<ParagraphProps>`
  font-size: ${props => (!props.isMobile ? '18px' : '14px')};
  letter-spacing: ${props => (!props.isMobile ? '-0.3px' : '-0.2px')};
`;

export const largeFontStyles = css<ParagraphProps>`
  font-size: ${props => (!props.isMobile ? '21px' : '14px')};
  letter-spacing: ${props => (!props.isMobile ? '-0.4px' : '-0.2px')};
`;

export const ParagraphStyled = styled.p<ParagraphProps>`
  ${ParagraphCommonStyles};
  ${props => props.size === 'sm' && smallFontStyles};
  ${props => props.size === 'md' && mediumFontStyles};
  ${props => props.size === 'lg' && largeFontStyles};
  margin: ${props => props.margin};
  padding: ${props => props.padding};
  width: ${props => (props.isFullWidth ? '100%' : 'auto')};
`;
