import React, { ReactNode } from 'react';
import styled, { css } from 'styled-components';
import { isEmpty } from 'lodash-es';
import { colorUtil } from '../../../../utils/ColorUtil';
import { textUtil } from '../../../../utils/TextUtil';
import theme from '../../../../theme';

export interface TitleProps {
  content?: string;
  size?: 'xl' | 'lg' | 'md' | 'sm';
  textAlign?: 'start' | 'center' | 'end';
  fontWeight?: 'normal' | 'bold';
  highlight?: {
    height: number;
    color: string;
    isVisible: boolean;
  };
  color?: string;
  partialColors?: string[];
  isMobile?: boolean;
  margin?: string;
  padding?: string;
  isVisible?: boolean;
  isFullWidth?: boolean;
}

function Title(props: TitleProps) {
  const { size, content, color, partialColors, isVisible, ...remain } = props;
  const texts = textUtil.processTextWithNewlines(content);

  if (!isVisible) return null;
  return (
    <TitleStyled {...props}>
      {texts?.map((text, i) => {
        const replacedText = colorUtil.applyColorsToText(text, partialColors);

        /** 텍스트가 가공된 경우 */
        if (!isEmpty(replacedText)) {
          const innerHtml = { __html: replacedText };
          return (
            <>
              <span key={i} dangerouslySetInnerHTML={innerHtml} />
              <br />
            </>
          );
        }

        /** 텍스트가 가공되지 않은 경우 */
        return (
          <>
            <span key={i}>{text}</span>
            <br />
          </>
        );
      })}
    </TitleStyled>
  );
}

Title.defaultProps = {
  size: 'xl',
  content: 'Title Title\nTitle Title Title',
  color: theme.colors.black,
  partialColors: [theme.colors.black],
  fontWeight: 'bold',
  highlight: {
    height: 28,
    color: theme.colors.gray500,
    isVisible: false,
  },
  isMobile: false,
  isVisible: true,
  isFullWidth: false,
  textAlign: 'left',
};

export default Title;

const highlightStyles = css<TitleProps>`
  display: inline;
  box-shadow: inset 0 -${props => props.highlight?.height}px 0 ${props => props.highlight?.color};
`;

const headingCommonStyles = css<TitleProps>`
  display: inline-block;
  text-align: ${props => props.textAlign};
  color: ${props => props.color};
  margin: 0;
  width: auto;
  height: auto;
  font-family: ${props => (props.fontWeight === 'bold' ? `${theme.fontFamily.bold}` : `${theme.fontFamily.normal}`)};
  font-weight: normal;
  span {
    ${props => props.highlight?.isVisible && highlightStyles}
  }
`;

const xlargeFontStyles = css<TitleProps>`
  font-size: ${props => (!props.isMobile ? '58px' : '36px')};
  letter-spacing: ${props => (!props.isMobile ? '-0.7px' : '-0.6px')};
  line-height: 1.25em;
`;

const largeFontStyles = css<TitleProps>`
  font-size: ${props => (!props.isMobile ? '42px' : '28px')};
  letter-spacing: ${props => (!props.isMobile ? '-0.6px' : '-0.5px')};
  line-height: 1.25em;
`;

const mediumFontStyles = css<TitleProps>`
  font-size: ${props => (!props.isMobile ? '32px' : '24px')};
  letter-spacing: ${props => (!props.isMobile ? '-0.5px' : '-0.4px')};
  line-height: 1.25em;
`;

const smallFontStyles = css<TitleProps>`
  font-size: ${props => (!props.isMobile ? '24px' : '18px')};
  letter-spacing: ${props => (!props.isMobile ? '-0.4px' : '-0.3px')};
  line-height: 1.25em;
`;

export const TitleStyled = styled.h1<TitleProps>`
  ${headingCommonStyles}
  ${props => props.size === 'xl' && xlargeFontStyles}
  ${props => props.size === 'lg' && largeFontStyles}
  ${props => props.size === 'md' && mediumFontStyles}
  ${props => props.size === 'sm' && smallFontStyles}
  margin: ${props => props.margin};
  padding: ${props => props.padding};
  width: ${props => (props.isFullWidth ? '100%' : 'auto')};
`;
