import styled from 'styled-components';
import { motion } from 'framer-motion';
import theme from '../../../../theme';
import Icon, { IconNames, iconNames } from '../../atoms/Icon';
import { isEmpty } from 'lodash-es';

export interface BadgeProps {
  round: 'sm' | 'md' | 'lg' | 'full';
  size: 'sm' | 'md' | 'lg';
  content: string;
  backgroundColor: string;
  color: string;
  icon: IconNames;
  iconPosition: 'left' | 'right';
  iconSize: number;
  isVisible?: boolean;
}

export default function Badge(props: BadgeProps) {
  const { content, icon, iconPosition, iconSize, color, size, ...remain } = props;
  const iconGap = size === 'sm' ? 2 : size === 'md' ? 4 : 6;

  if (!props?.isVisible) return null;
  return (
    <BadgeStyled {...props}>
      {!isEmpty(icon) && iconPosition === 'left' && <Icon name={icon} size={iconSize} color={color} />}
      {content && (
        <TextStyled gap={iconGap} icon={icon} position={iconPosition} size={size} color={color}>
          {content}
        </TextStyled>
      )}
      {!isEmpty(icon) && iconPosition === 'right' && <Icon name={icon} size={iconSize} color={color} />}
    </BadgeStyled>
  );
}

Badge.defaultProps = {
  size: 'md',
  round: 'lg',
  content: 'Badge',
  backgroundColor: theme.colors.gray50,
  color: theme.colors.gray500,
  icon: null,
  iconPosition: 'left',
  iconSize: 16,
  isVisible: true,
};

const BadgeStyled = styled(motion.div)<BadgeProps>`
  display: inline-flex;
  justify-content: center;
  align-items: center;
  font-family: 'Pretendard Regular';
  color: ${props => props.color};
  background-color: ${props => props.backgroundColor};
  border-radius: ${({ round }) => (round === 'full' ? '50%' : round === 'sm' ? '4px' : round === 'md' ? '8px' : round === 'lg' ? '12px' : '0px')};
  border: none;
  ${props =>
    props.size === 'sm' &&
    `
        font-size: 14px;
        line-height: 20px;
        letter-spacing: -0.2px;
        padding: ${props.round === 'full' ? '0' : '2px 6px'};
        width: ${props.round === 'full' ? '28px' : 'auto'};
        height: ${props.round === 'full' ? '28px' : 'auto'};
    `}

  ${props =>
    props.size === 'md' &&
    `
        font-size: 16px;
        line-height: 24px;
        letter-spacing: -0.3px;
        padding: ${props.round === 'full' ? '4px' : '4px 8px'};
        width: ${props.round === 'full' ? '32px' : 'auto'};
        height: ${props.round === 'full' ? '32px' : 'auto'};
    `}

    ${props =>
    props.size === 'lg' &&
    `
        font-size: 24px;
        line-height: 32px;
        letter-spacing: -0.4px;
        padding: ${props.round === 'full' ? '0' : '8px 12px'};
        width: ${props.round === 'full' ? '48px' : 'auto'};
        height: ${props.round === 'full' ? '48px' : 'auto'};
    `}
`;

const TextStyled = styled.span<any>`
  color: inherit;
  margin-left: ${props => (props.icon && props.position === 'left' ? props.gap : 0)}px;
  margin-right: ${props => (props.icon && props.position === 'right' ? props.gap : 0)}px;
`;
