import styled, { css } from 'styled-components';
import theme from '../../../../theme';
import Icon, { IconNames, iconNames } from '../../atoms/Icon';
import { isEmpty } from 'lodash-es';

export interface ButtonProps {
  buttonType?: 'solid' | 'outline' | 'ghost' | 'link';
  icon?: IconNames;
  iconPosition?: 'left' | 'right';
  size?: 'sm' | 'md' | 'lg' | 'xl';
  backgroundColor?: string;
  color?: string;
  border?: {
    color?: string;
    width?: number;
    radius?: number;
  };
  content?: string;
  isLoading?: boolean;
  isDisabled?: boolean;
  isFullWidth?: boolean;
  onClick?: any;
  margin?: string;
}

export default function Button(props: ButtonProps) {
  const { content, icon, size, color, iconPosition, isLoading, isDisabled, onClick, ...remain } = props;
  const iconGap = size === 'sm' ? 4 : size === 'md' ? 8 : size === 'lg' ? 8 : 12;
  const iconSize = size === 'sm' ? 16 : 24;

  if (isLoading) return <ButtonStyled {...props}>Loading...</ButtonStyled>;
  return (
    <ButtonStyled onClick={onClick} disabled={isDisabled} {...props}>
      {!isEmpty(icon) && iconPosition === 'left' && <Icon name={icon} size={iconSize} color={color} />}
      {content && (
        <TextStyled gap={iconGap} icon={icon} position={iconPosition}>
          {content}
        </TextStyled>
      )}
      {!isEmpty(icon) && iconPosition === 'right' && <Icon name={icon} size={iconSize} color={color} />}
    </ButtonStyled>
  );
}

Button.defaultProps = {
  buttonType: 'solid',
  size: 'md',
  content: 'Button',
  backgroundColor: theme.colors.black,
  color: theme.colors.white,
  border: {
    width: 0,
    radius: 12,
    color: null,
  },
  icon: 'home',
  iconPosition: 'left',
  isLoading: false,
  isDisabled: false,
  isFullWidth: false,
};

const buttonBlackLayerStyle = css<ButtonProps>`
  content: '';
  position: absolute;
  top: 0;
  left: 0;
  width: 100%;
  height: 100%;
  background-color: rgba(0, 0, 0, 0.2);
  border-radius: ${props => props.border?.radius}px;
`;

const solidStyle = css<ButtonProps>`
  border: none;
  background: ${props => props.backgroundColor};
  position: relative;

  &:hover {
    &::after {
      ${buttonBlackLayerStyle}
    }
  }

  &:disabled {
    background: none;
    background-color: ${theme.colors.gray200};
    color: ${theme.colors.gray500};
  }
`;

const ghostStyle = css<ButtonProps>`
  border-width: 0px;
  position: relative;
  background-color: transparent;

  &:hover {
    &::after {
      ${buttonBlackLayerStyle}
    }
  }

  &:disabled {
    background-color: ${theme.colors.gray200};
    color: ${theme.colors.gray500};
  }
`;

const linkStyle = css<ButtonProps>`
  padding: 0;
  background-color: transparent;
  cursor: pointer;
  border-width: 0px;

  ${props =>
    props.size === 'xl' &&
    `
        font-size: 24px;
        letter-spacing: -0.4px;
        line-height: 32px;
    `};
  ${props =>
    props.size === 'lg' &&
    `
        font-size: 18px;
        letter-spacing: -0.3px;
        line-height: 28px;
    `};
  ${props =>
    props.size === 'md' &&
    `
        font-size: 16px;
        letter-spacing: -0.4px;
        line-height: 24px;
    `};
  ${props =>
    props.size === 'sm' &&
    `
        font-size: 12px;
        letter-spacing: -0.2px;
        line-height: 16px;
    `};

  &:hover {
    opacity: 0.6;
  }

  &:disabled {
    opacity: 0.3;
  }
`;

const outlineStyle = css<ButtonProps>`
  position: relative;
  border-buttontype: solid;
  border-width: ${props => props.border?.width || 1}px;
  border-radius: ${props => props.border?.radius || 12}px;
  border-color: ${props => props.color || theme.colors.black};
  color: ${props => props.color || theme.colors.black};
  background-color: transparent;

  &:hover {
    &::after {
      ${buttonBlackLayerStyle}
    }
  }

  &:disabled {
    background-color: ${theme.colors.gray200};
    color: ${theme.colors.gray500};
    border-color: ${theme.colors.gray500};
  }
`;

const ButtonStyled = styled.button<ButtonProps>`
  display: flex;
  justify-content: center;
  align-items: center;
  font-family: 'Pretendard Bold';
  color: ${props => props.color};
  background: none;
  cursor: pointer;
  border-radius: 12px;
  width: ${props => (props.isFullWidth ? '100%' : 'auto')};

  margin: ${props => props.margin};

  ${props =>
    props.size === 'sm' &&
    `
        padding: 8px 12px;
        font-size: 12px;
        line-height: 16px;
        letter-spacing: -0.2px;
    `};

  ${props =>
    props.size === 'md' &&
    `
        padding: 12px 20px;
        font-size: 16px;
        line-height: 24px;
        letter-spacing: -0.3px;
    `};

  ${props =>
    props.size === 'lg' &&
    `
        padding: 15px 24px;
        font-size: 18px;
        line-height: 26px;
        letter-spacing: -0.3px;
    `};

  ${props =>
    props.size === 'xl' &&
    `
        padding: 19px 28px;
        font-size: 18px;
        line-height: 26px;
        letter-spacing: -0.3px;
    `};

  ${props => props.buttonType === 'solid' && solidStyle};
  ${props => props.buttonType === 'ghost' && ghostStyle};
  ${props => props.buttonType === 'link' && linkStyle};
  ${props => props.buttonType === 'outline' && outlineStyle};
`;

const TextStyled = styled.span<any>`
  color: inherit;
  white-space: nowrap;
  margin-left: ${props => (props.icon && props.position === 'left' ? props.gap : 0)}px;
  margin-right: ${props => (props.icon && props.position === 'right' ? props.gap : 0)}px;
`;
