import * as React from 'react';
import styled from 'styled-components';
import { isEmpty } from 'lodash-es';
import { Stack } from '../../../layout';
import Button from '../../molecules/Button/Button';
import theme from '../../../../theme';

export interface HeaderProps {
  direction?: 'row' | 'column';
  menus?: { text: string; href: string; isActive: boolean }[];
  menuColor?: string; // 메뉴 컬러
  menuGap?: number; // 메뉴 사이의 간격
  menuButtonGap?: number; // 메뉴와 버튼 사이 간격
  borderBottomColor?: string; // 헤더 하단 컬러
  logo?: string; // 로고 URL
  logoHeight?: number; // 로고 높이 (너비는 높이에 의해서 결정됨)
  headerBackground: string; // 헤더 배경
  isMobile?: boolean;
  buttons: React.ReactNode[]; // 버튼 목록
}

export default function Header(props: HeaderProps) {
  const { menus, menuColor, menuGap, menuButtonGap, borderBottomColor, logo, logoHeight, headerBackground, buttons, isMobile } = props;
  const direction = !isMobile ? 'row' : 'column';

  return (
    <Box borderBottomColor={borderBottomColor} headerBackground={headerBackground} isMobile={isMobile}>
      <LeftArea>
        <Logo src={logo} alt="로고" height={logoHeight} />
      </LeftArea>
      <RightArea>
        {!isMobile && (
          <Stack direction={direction} gap={menuGap}>
            {menus.map((menu, index) => {
              const { text, href, isActive } = menu;
              return (
                <Menu key={index} href={href} menuColor={menuColor} isActive={isActive}>
                  {text}
                </Menu>
              );
            })}
          </Stack>
        )}
        {!isEmpty(buttons) && (
          <RightButtonArea menuButtonGap={menuButtonGap}>
            <Stack direction="row" gap={24} alignItems="center">
              {buttons}
            </Stack>
          </RightButtonArea>
        )}
      </RightArea>
    </Box>
  );
}

Header.defaultProps = {
  borderBottomColor: '#8D8D8D',
  headerBackground: '#f9f9f9',
  menuColor: theme.colors.black,
  menus: [
    { text: '메뉴1', href: null, isActive: true },
    { text: '메뉴2', href: null, isActive: true },
    { text: '메뉴3', href: null, isActive: true },
  ],
  menuGap: 56,
  menuButtonGap: 32,
  logo: 'https://codeindesign.s3.ap-northeast-2.amazonaws.com/CodeInDesign+Logo.png',
  logoHeight: 32,
  buttons: [<Button />, <Button />],
  isMobile: false,
};

const Box = styled.div<any>`
  width: 100%;
  padding: ${props => (!props.isMobile ? '17px 32px' : '19px 16px')};
  display: flex;
  justify-content: space-between;
  align-items: center;
  background-color: ${props => props.headerBackground};
  border-bottom: 1px solid ${props => props.borderBottomColor};
`;

const Menu = styled.a<any>`
  cursor: pointer;
  color: ${props => props.menuColor};
  text-decoration: none;
  font-size: 18px;
  font-family: 'Pretendard Regular';
  font-weight: ${props => (props.isActive ? 'bold' : 'normal')};

  &:hover {
    color: white;
  }
`;

const RightArea = styled.div`
  display: flex;
  align-items: center;
`;

const RightButtonArea = styled.div<any>`
  display: flex;
  height: 100%;
  align-items: center;
  margin-left: ${props => props.menuButtonGap}px;
`;

const LeftArea = styled.div`
  display: flex;
  align-items: center;
`;

const Logo = styled.img<any>`
  height: ${props => props.logoHeight}px;
  object-fit: contain;
`;
