import styled from 'styled-components';
import { motion } from 'framer-motion';
import { isEmpty } from 'lodash-es';
import { produce } from 'immer';
import theme from '../../../../theme';
import { Container, Stack } from '../../../layout';
import Paragraph, { ParagraphProps } from '../../atoms/Paragraph/Paragraph';
import Title, { TitleProps } from '../../atoms/Title/Title';
import Badge, { BadgeProps } from '../../molecules/Badge/Badge';
import Button, { ButtonProps } from '../../molecules/Button/Button';

export interface FeatureSection1Props {
  imageUrls: string[];
  badgeProps: BadgeProps;
  subTitleProps: TitleProps;
  titleProps: TitleProps;
  paragraphProps: ParagraphProps;
  imageUrl?: string;
  buttonsProps: ButtonProps[];
  backgroundImage?: string;
  backgroundColor?: string;
  isMobile?: boolean;
  isCenter?: boolean;
}

export default function FeatureSection1(props: FeatureSection1Props) {
  const { imageUrls, badgeProps, subTitleProps, titleProps, paragraphProps, imageUrl, buttonsProps, backgroundImage, backgroundColor, isMobile, isCenter } = props;

  return (
    <Container isMobile={isMobile} direction="column" isCenter={isCenter} backgroundColor={backgroundColor} backgroundImage={backgroundImage}>
      <Stack direction="column" gap={24} alignItems="center">
        {!isEmpty(imageUrls) && (
          <Stack gap={8} justifyContent="center">
            {imageUrls?.map((imageUrl, index) => {
              return <IconImage key={index} src={imageUrl} />;
            })}
          </Stack>
        )}
        {badgeProps.isVisible && (
          <Box>
            <Badge {...badgeProps} />
          </Box>
        )}
        {subTitleProps.isVisible && (
          <Box>
            <Title {...subTitleProps} isMobile={isMobile} />
          </Box>
        )}
      </Stack>
      {titleProps.isVisible && (
        <Box margin="0 0 8px">
          <Title {...titleProps} isMobile={isMobile} />
        </Box>
      )}
      {paragraphProps.isVisible && (
        <Box>
          <Paragraph {...paragraphProps} isMobile={isMobile} />
        </Box>
      )}
      <Box width="100%" margin={isMobile ? '32px 0 0' : '64px 0 0'}>
        <FeatureImage src={imageUrl} />
      </Box>
      {!isEmpty(buttonsProps) && (
        <Stack gap={10} direction={isMobile ? 'column' : 'row'} justifyContent="center" margin={isMobile ? '32px 0 0' : '64px 0 0'}>
          {buttonsProps?.map((button, index) => {
            return (
              <ButtonBox>
                <Button key={index} {...button} size={isMobile ? 'md' : 'xl'} />
              </ButtonBox>
            );
          })}
        </Stack>
      )}
    </Container>
  );
}

FeatureSection1.defaultProps = {
  imageUrl: 'https://picsum.photos/id/347/1200',
  imageUrls: ['https://picsum.photos/id/1/32', 'https://picsum.photos/id/2/32', 'https://picsum.photos/id/3/32'],
  badgeProps: Badge.defaultProps,
  subTitleProps: produce(Title.defaultProps, draft => {
    draft.content = 'Subtitle 01 서브타이틀 01';
    draft.size = 'sm';
  }),
  titleProps: produce(Title.defaultProps, draft => {
    draft.content = 'Title 01 타이틀 01\nTitle 01 타이틀 01';
    draft.size = 'lg';
  }),
  paragraphProps: produce(Paragraph.defaultProps, draft => {
    draft.content = 'Body 입력 Body 입력 무엇을 구하지 꽃이 무엇이 천고에 천자만홍이 황금시대다. 피는 대한 따뜻한 꽃 가슴이 봄바람이다.속에서 밥을 끓는 속잎나고.';
    draft.size = 'lg';
    draft.color = theme.colors.gray500;
  }),
  buttonsProps: Array(1).fill({ ...Button.defaultProps, isFullWidth: true, size: 'xl' }),
  backgroundImage: null,
  backgroundColor: 'transparent',
  isMobile: false,
  isCenter: true,
};

const Box = styled(motion.div)<any>`
  width: ${props => props.width};
  margin: ${props => props.margin};
`;

const ButtonBox = styled(motion.div)<any>`
  flex: 1;
  width: 100%;
`;

const FeatureImage = styled(motion.img)<any>`
  width: 100%;
  border-radius: 20px;
  overflow: hidden;
`;

const IconImage = styled(motion.img)<any>`
  height: ${props => (props.isMobile ? '32px' : '40px')};
`;
