import styled from 'styled-components';
import { motion } from 'framer-motion';
import { produce } from 'immer';
import { isEmpty } from 'lodash-es';
import theme from '../../../../theme';
import { Container, Stack } from '../../../layout';
import Paragraph, { ParagraphProps } from '../../atoms/Paragraph/Paragraph';
import Title, { TitleProps } from '../../atoms/Title/Title';
import Badge, { BadgeProps } from '../../molecules/Badge/Badge';
import Button, { ButtonProps } from '../../molecules/Button/Button';
import Feature1, { Feature1Props } from '../../organisms/Feature1/Feature1';

export interface FeatureSection3Props {
  imageUrls: string[];
  badgeProps: BadgeProps;
  subTitleProps: TitleProps;
  titleProps: TitleProps;
  featuresProps: Feature1Props[];
  paragraphProps: ParagraphProps;
  buttonsProps: ButtonProps[];
  backgroundImage?: string;
  backgroundColor?: string;
  isMobile?: boolean;
}

export default function FeatureSection3(props: FeatureSection3Props) {
  const { imageUrls, badgeProps, subTitleProps, titleProps, featuresProps, paragraphProps, buttonsProps, backgroundImage, backgroundColor, isMobile } = props;

  return (
    <Container isMobile={isMobile} direction="column" isCenter={true} backgroundColor={backgroundColor} backgroundImage={backgroundImage}>
      <Stack direction="column" gap={24} alignItems="center">
        {!isEmpty(imageUrls) && (
          <Stack gap={8} justifyContent="center">
            {imageUrls?.map((imageUrl, index) => {
              return <IconImage key={index} src={imageUrl} isMobile={isMobile} />;
            })}
          </Stack>
        )}
        <Badge {...badgeProps} />
        <Title {...subTitleProps} isMobile={isMobile} />
      </Stack>
      <Title {...titleProps} isMobile={isMobile} margin="8px 0 0" />
      <Paragraph {...paragraphProps} isMobile={isMobile} margin="8px 0 0" />
      <FeaturesBox isMobile={isMobile}>
        {featuresProps?.map((feature, index) => {
          return <Feature1 key={index} {...feature} isMobile={isMobile} />;
        })}
      </FeaturesBox>
      {!isEmpty(buttonsProps) && (
        <Stack gap={10} direction={isMobile ? 'column' : 'row'} justifyContent="center" margin={isMobile ? '32px 0 0' : '64px 0 0'}>
          {buttonsProps?.map((button, index) => {
            return (
              <ButtonBox>
                <Button key={index} {...button} size={isMobile ? 'md' : 'xl'} />
              </ButtonBox>
            );
          })}
        </Stack>
      )}
    </Container>
  );
}

FeatureSection3.defaultProps = {
  imageUrls: ['https://picsum.photos/id/1/32', 'https://picsum.photos/id/2/32', 'https://picsum.photos/id/3/32'],
  badgeProps: Badge.defaultProps,
  subTitleProps: produce(Title.defaultProps, draft => {
    draft.content = 'Subtitle 01 서브타이틀 01';
    draft.size = 'sm';
    draft.textAlign = 'center';
  }),
  titleProps: produce(Title.defaultProps, draft => {
    draft.content = 'Title 01 타이틀 01\nTitle 01 타이틀 01';
    draft.size = 'lg';
    draft.textAlign = 'center';
  }),
  paragraphProps: produce(Paragraph.defaultProps, draft => {
    draft.content = 'Body 입력 Body 입력 무엇을 구하지 꽃이 무엇이 천고에 천자만홍이 황금시대다. 피는 대한 따뜻한 꽃 가슴이 봄바람이다.속에서 밥을 끓는 속잎나고.';
    draft.size = 'md';
    draft.textAlign = 'center';
    draft.color = theme.colors.gray500;
  }),
  featuresProps: Array(3).fill(Feature1.defaultProps),
  buttonsProps: Array(1).fill(
    produce(Button.defaultProps, draft => {
      draft.isFullWidth = true;
      draft.size = 'xl';
    }),
  ),
  description: produce(Paragraph.defaultProps, draft => {
    draft.content = 'Body 입력 Body 입력 무엇을 구하지 꽃이 무엇이 천고에 천자만홍이 황금시대다.\n피는 대한 따뜻한 꽃 가슴이 봄바람이다. 속에서 밥을 끓는 속잎나고.';
    draft.textAlign = 'center';
  }),
  backgroundImage: 'https://picsum.photos/id/347/1200',
  backgroundColor: 'transparent',
  isMobile: false,
};

const Box = styled(motion.div)<any>`
  display: flex;
  justify-content: center;
  align-items: center;
  background: url(${props => props.backgroundImage}) no-repeat;
  background-size: cover;
  background-color: ${props => props.backgroundColor};
`;

const ButtonBox = styled(motion.div)<any>`
  flex: 1;
  width: 100%;
`;

const FeaturesBox = styled(motion.div)<any>`
  display: flex;
  margin-top: 64px;
  gap: 24px;

  ${props =>
    props.isMobile &&
    `
        flex-direction: column;
        gap: 32px;
        margin-top: 32px;
    `};
`;

const IconImage = styled(motion.img)<any>`
  height: ${props => (props.isMobile ? '32px' : '40px')};
`;
